/*
 * This file is a component of thundr, a software library from 3wks.
 * Read more: http://www.3wks.com.au/thundr
 * Copyright (C) 2013 3wks, <thundr@3wks.com.au>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.threewks.thundr.view.handlebars;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import javax.servlet.ServletContext;

import com.github.jknack.handlebars.TemplateLoader;

public class CompositeTemplateLoader extends TemplateLoader {

	private ServletContext servletContext;

	public CompositeTemplateLoader(ServletContext servletContext) {
		this.servletContext = servletContext;
		setPrefix("");
		setSuffix("");
	}

	@Override
	protected Reader read(String location) throws IOException {
		Reader reader = readFromContainer(location);
		if (reader == null) {
			reader = readFromClasspath(location);
		}
		return reader;
	}

	public URI getResource(String location) {
		location = resolve(location);
		try {
			URI resource = getResourceFromContainer(location);
			if (resource == null) {
				resource = getResourceFromClasspath(location);
			}
			return resource;
		} catch (Exception e) {
			return null;
		}
	}

	private URI getResourceFromContainer(String location) throws MalformedURLException, URISyntaxException {
		URL resource = servletContext.getResource(location);
		return resource == null ? null : resource.toURI();
	}

	private URI getResourceFromClasspath(String location) throws MalformedURLException, URISyntaxException {
		URL resource = this.getClass().getResource(location);
		return resource == null ? null : resource.toURI();
	}

	private Reader readFromContainer(String location) throws UnsupportedEncodingException {
		InputStream input = servletContext.getResourceAsStream(location);
		if (input == null) {
			return null;
		}
		return new InputStreamReader(input, "UTF-8");
	}

	private Reader readFromClasspath(String location) throws UnsupportedEncodingException {
		InputStream input = getClass().getResourceAsStream(location);
		if (input == null) {
			return null;
		}
		return new InputStreamReader(input, "UTF-8");
	}

}
